/****************************************************************************
** Meta object code from reading C++ file 'kgameprocess.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kfourinline-26.07.90/src/kgameprocess.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kgameprocess.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16KMessageFilePipeE_t {};
} // unnamed namespace

template <> constexpr inline auto KMessageFilePipe::qt_create_metaobjectdata<qt_meta_tag_ZN16KMessageFilePipeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KMessageFilePipe"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KMessageFilePipe, qt_meta_tag_ZN16KMessageFilePipeE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KMessageFilePipe::staticMetaObject = { {
    QMetaObject::SuperData::link<KMessageIO::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16KMessageFilePipeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16KMessageFilePipeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16KMessageFilePipeE_t>.metaTypes,
    nullptr
} };

void KMessageFilePipe::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KMessageFilePipe *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KMessageFilePipe::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KMessageFilePipe::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16KMessageFilePipeE_t>.strings))
        return static_cast<void*>(this);
    return KMessageIO::qt_metacast(_clname);
}

int KMessageFilePipe::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KMessageIO::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN12KGameProcessE_t {};
} // unnamed namespace

template <> constexpr inline auto KGameProcess::qt_create_metaobjectdata<qt_meta_tag_ZN12KGameProcessE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KGameProcess",
        "signalCommand",
        "",
        "QDataStream&",
        "inputStream",
        "msgid",
        "receiver",
        "sender",
        "signalTurn",
        "stream",
        "turn",
        "signalInit",
        "userid",
        "receivedMessage",
        "receiveBuffer"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'signalCommand'
        QtMocHelpers::SignalData<void(QDataStream &, int, int, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 5 }, { QMetaType::Int, 6 }, { QMetaType::Int, 7 },
        }}),
        // Signal 'signalTurn'
        QtMocHelpers::SignalData<void(QDataStream &, bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 9 }, { QMetaType::Bool, 10 },
        }}),
        // Signal 'signalInit'
        QtMocHelpers::SignalData<void(QDataStream &, int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 9 }, { QMetaType::Int, 12 },
        }}),
        // Slot 'receivedMessage'
        QtMocHelpers::SlotData<void(const QByteArray &)>(13, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QByteArray, 14 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KGameProcess, qt_meta_tag_ZN12KGameProcessE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KGameProcess::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KGameProcessE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KGameProcessE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12KGameProcessE_t>.metaTypes,
    nullptr
} };

void KGameProcess::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KGameProcess *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->signalCommand((*reinterpret_cast<std::add_pointer_t<QDataStream&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 1: _t->signalTurn((*reinterpret_cast<std::add_pointer_t<QDataStream&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 2: _t->signalInit((*reinterpret_cast<std::add_pointer_t<QDataStream&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 3: _t->receivedMessage((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KGameProcess::*)(QDataStream & , int , int , int )>(_a, &KGameProcess::signalCommand, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KGameProcess::*)(QDataStream & , bool )>(_a, &KGameProcess::signalTurn, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KGameProcess::*)(QDataStream & , int )>(_a, &KGameProcess::signalInit, 2))
            return;
    }
}

const QMetaObject *KGameProcess::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KGameProcess::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KGameProcessE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KGameProcess::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void KGameProcess::signalCommand(QDataStream & _t1, int _t2, int _t3, int _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 1
void KGameProcess::signalTurn(QDataStream & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void KGameProcess::signalInit(QDataStream & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}
QT_WARNING_POP
