/****************************************************************************
** Meta object code from reading C++ file 'kwin4view.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kfourinline-26.07.90/src/kwin4view.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kwin4view.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9KWin4ViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KWin4View::qt_create_metaobjectdata<qt_meta_tag_ZN9KWin4ViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KWin4View",
        "signalMoveDone",
        "",
        "mode",
        "signalQuickStart",
        "COLOUR",
        "startPlayer",
        "KGameIO::IOMode",
        "input0",
        "input1",
        "aiLevel",
        "updateAndAdvance",
        "mouseInput",
        "KGameIO*",
        "input",
        "QDataStream&",
        "stream",
        "QMouseEvent*",
        "mouse",
        "bool*",
        "eatevent",
        "keyInput",
        "QKeyEvent*",
        "key",
        "moveDone",
        "QGraphicsItem*",
        "item",
        "rescaleTheme"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'signalMoveDone'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'signalQuickStart'
        QtMocHelpers::SignalData<void(COLOUR, KGameIO::IOMode, KGameIO::IOMode, int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 7, 8 }, { 0x80000000 | 7, 9 }, { QMetaType::Int, 10 },
        }}),
        // Slot 'updateAndAdvance'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'mouseInput'
        QtMocHelpers::SlotData<void(KGameIO *, QDataStream &, QMouseEvent *, bool *)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { 0x80000000 | 15, 16 }, { 0x80000000 | 17, 18 }, { 0x80000000 | 19, 20 },
        }}),
        // Slot 'keyInput'
        QtMocHelpers::SlotData<void(KGameIO *, QDataStream &, QKeyEvent *, bool *)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { 0x80000000 | 15, 16 }, { 0x80000000 | 22, 23 }, { 0x80000000 | 19, 20 },
        }}),
        // Slot 'moveDone'
        QtMocHelpers::SlotData<void(QGraphicsItem *, int)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 26 }, { QMetaType::Int, 3 },
        }}),
        // Slot 'rescaleTheme'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KWin4View, qt_meta_tag_ZN9KWin4ViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KWin4View::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9KWin4ViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9KWin4ViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9KWin4ViewE_t>.metaTypes,
    nullptr
} };

void KWin4View::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KWin4View *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->signalMoveDone((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->signalQuickStart((*reinterpret_cast<std::add_pointer_t<COLOUR>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KGameIO::IOMode>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KGameIO::IOMode>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 2: _t->updateAndAdvance(); break;
        case 3: _t->mouseInput((*reinterpret_cast<std::add_pointer_t<KGameIO*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDataStream&>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QMouseEvent*>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool*>>(_a[4]))); break;
        case 4: _t->keyInput((*reinterpret_cast<std::add_pointer_t<KGameIO*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDataStream&>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QKeyEvent*>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool*>>(_a[4]))); break;
        case 5: _t->moveDone((*reinterpret_cast<std::add_pointer_t<QGraphicsItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 6: _t->rescaleTheme(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KGameIO* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KGameIO* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGraphicsItem* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KWin4View::*)(int )>(_a, &KWin4View::signalMoveDone, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KWin4View::*)(COLOUR , KGameIO::IOMode , KGameIO::IOMode , int )>(_a, &KWin4View::signalQuickStart, 1))
            return;
    }
}

const QMetaObject *KWin4View::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KWin4View::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9KWin4ViewE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Themeable"))
        return static_cast< Themeable*>(this);
    return QGraphicsView::qt_metacast(_clname);
}

int KWin4View::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void KWin4View::signalMoveDone(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KWin4View::signalQuickStart(COLOUR _t1, KGameIO::IOMode _t2, KGameIO::IOMode _t3, int _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3, _t4);
}
QT_WARNING_POP
