/****************************************************************************
** Meta object code from reading C++ file 'displayintro.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kfourinline-26.07.90/src/displayintro.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'displayintro.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12DisplayIntroE_t {};
} // unnamed namespace

template <> constexpr inline auto DisplayIntro::qt_create_metaobjectdata<qt_meta_tag_ZN12DisplayIntroE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DisplayIntro",
        "signalQuickStart",
        "",
        "COLOUR",
        "startPlayer",
        "KGameIO::IOMode",
        "input0",
        "input1",
        "aiLevel",
        "advance",
        "buttonPressed",
        "QGraphicsItem*",
        "item",
        "id",
        "findSprite",
        "QPoint",
        "pos"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'signalQuickStart'
        QtMocHelpers::SignalData<void(COLOUR, KGameIO::IOMode, KGameIO::IOMode, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 }, { 0x80000000 | 5, 7 }, { QMetaType::Int, 8 },
        }}),
        // Slot 'advance'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'buttonPressed'
        QtMocHelpers::SlotData<void(QGraphicsItem *, int)>(10, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { QMetaType::Int, 13 },
        }}),
        // Slot 'findSprite'
        QtMocHelpers::SlotData<QGraphicsItem *(QPoint)>(14, 2, QMC::AccessProtected, 0x80000000 | 11, {{
            { 0x80000000 | 15, 16 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DisplayIntro, qt_meta_tag_ZN12DisplayIntroE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DisplayIntro::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12DisplayIntroE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12DisplayIntroE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12DisplayIntroE_t>.metaTypes,
    nullptr
} };

void DisplayIntro::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DisplayIntro *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->signalQuickStart((*reinterpret_cast<std::add_pointer_t<COLOUR>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KGameIO::IOMode>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KGameIO::IOMode>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 1: _t->advance(); break;
        case 2: _t->buttonPressed((*reinterpret_cast<std::add_pointer_t<QGraphicsItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 3: { QGraphicsItem* _r = _t->findSprite((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QGraphicsItem**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGraphicsItem* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DisplayIntro::*)(COLOUR , KGameIO::IOMode , KGameIO::IOMode , int )>(_a, &DisplayIntro::signalQuickStart, 0))
            return;
    }
}

const QMetaObject *DisplayIntro::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DisplayIntro::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12DisplayIntroE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Themeable"))
        return static_cast< Themeable*>(this);
    return QObject::qt_metacast(_clname);
}

int DisplayIntro::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void DisplayIntro::signalQuickStart(COLOUR _t1, KGameIO::IOMode _t2, KGameIO::IOMode _t3, int _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3, _t4);
}
QT_WARNING_POP
