/********************************************************************************
** Form generated from reading UI file 'statistics.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_STATISTICS_H
#define UI_STATISTICS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLCDNumber>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_Statistics
{
public:
    QGridLayout *gridLayout;
    QFrame *line1;
    QSpacerItem *spacerItem;
    QLCDNumber *p2_lost;
    QLabel *p1_name;
    QLabel *Name;
    QLabel *won;
    QLCDNumber *p2_drawn;
    QLabel *lost;
    QLCDNumber *p2_aborted;
    QLCDNumber *p2_sum;
    QLabel *sum;
    QLabel *aborted;
    QHBoxLayout *hboxLayout;
    QPushButton *pushButton1;
    QSpacerItem *spacerItem1;
    QPushButton *pushButton2;
    QLCDNumber *p1_won;
    QLabel *p2_name;
    QLCDNumber *p2_won;
    QLabel *drawn;
    QLCDNumber *p1_drawn;
    QLCDNumber *p1_lost;
    QLCDNumber *p1_aborted;
    QLCDNumber *p1_sum;

    void setupUi(QDialog *Statistics)
    {
        if (Statistics->objectName().isEmpty())
            Statistics->setObjectName("Statistics");
        Statistics->resize(439, 201);
        gridLayout = new QGridLayout(Statistics);
        gridLayout->setObjectName("gridLayout");
        line1 = new QFrame(Statistics);
        line1->setObjectName("line1");
        line1->setFrameShape(QFrame::HLine);
        line1->setFrameShadow(QFrame::Sunken);
        line1->setFrameShape(QFrame::Shape::HLine);

        gridLayout->addWidget(line1, 1, 0, 1, 6);

        spacerItem = new QSpacerItem(20, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(spacerItem, 5, 2, 1, 1);

        p2_lost = new QLCDNumber(Statistics);
        p2_lost->setObjectName("p2_lost");

        gridLayout->addWidget(p2_lost, 4, 3, 1, 1);

        p1_name = new QLabel(Statistics);
        p1_name->setObjectName("p1_name");

        gridLayout->addWidget(p1_name, 2, 0, 2, 1);

        Name = new QLabel(Statistics);
        Name->setObjectName("Name");
        Name->setAlignment(Qt::AlignCenter);

        gridLayout->addWidget(Name, 0, 0, 1, 1);

        won = new QLabel(Statistics);
        won->setObjectName("won");
        won->setAlignment(Qt::AlignCenter);

        gridLayout->addWidget(won, 0, 1, 1, 1);

        p2_drawn = new QLCDNumber(Statistics);
        p2_drawn->setObjectName("p2_drawn");

        gridLayout->addWidget(p2_drawn, 4, 2, 1, 1);

        lost = new QLabel(Statistics);
        lost->setObjectName("lost");
        lost->setAlignment(Qt::AlignCenter);

        gridLayout->addWidget(lost, 0, 3, 1, 1);

        p2_aborted = new QLCDNumber(Statistics);
        p2_aborted->setObjectName("p2_aborted");

        gridLayout->addWidget(p2_aborted, 4, 4, 1, 1);

        p2_sum = new QLCDNumber(Statistics);
        p2_sum->setObjectName("p2_sum");

        gridLayout->addWidget(p2_sum, 4, 5, 1, 1);

        sum = new QLabel(Statistics);
        sum->setObjectName("sum");
        sum->setAlignment(Qt::AlignCenter);

        gridLayout->addWidget(sum, 0, 5, 1, 1);

        aborted = new QLabel(Statistics);
        aborted->setObjectName("aborted");
        aborted->setAlignment(Qt::AlignCenter);

        gridLayout->addWidget(aborted, 0, 4, 1, 1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        pushButton1 = new QPushButton(Statistics);
        pushButton1->setObjectName("pushButton1");

        hboxLayout->addWidget(pushButton1);

        spacerItem1 = new QSpacerItem(51, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem1);

        pushButton2 = new QPushButton(Statistics);
        pushButton2->setObjectName("pushButton2");

        hboxLayout->addWidget(pushButton2);


        gridLayout->addLayout(hboxLayout, 6, 0, 1, 6);

        p1_won = new QLCDNumber(Statistics);
        p1_won->setObjectName("p1_won");

        gridLayout->addWidget(p1_won, 3, 1, 1, 1);

        p2_name = new QLabel(Statistics);
        p2_name->setObjectName("p2_name");

        gridLayout->addWidget(p2_name, 4, 0, 1, 1);

        p2_won = new QLCDNumber(Statistics);
        p2_won->setObjectName("p2_won");

        gridLayout->addWidget(p2_won, 4, 1, 1, 1);

        drawn = new QLabel(Statistics);
        drawn->setObjectName("drawn");
        drawn->setAlignment(Qt::AlignCenter);

        gridLayout->addWidget(drawn, 0, 2, 1, 1);

        p1_drawn = new QLCDNumber(Statistics);
        p1_drawn->setObjectName("p1_drawn");

        gridLayout->addWidget(p1_drawn, 3, 2, 1, 1);

        p1_lost = new QLCDNumber(Statistics);
        p1_lost->setObjectName("p1_lost");

        gridLayout->addWidget(p1_lost, 3, 3, 1, 1);

        p1_aborted = new QLCDNumber(Statistics);
        p1_aborted->setObjectName("p1_aborted");

        gridLayout->addWidget(p1_aborted, 3, 4, 1, 1);

        p1_sum = new QLCDNumber(Statistics);
        p1_sum->setObjectName("p1_sum");

        gridLayout->addWidget(p1_sum, 3, 5, 1, 1);

        QWidget::setTabOrder(pushButton2, pushButton1);

        retranslateUi(Statistics);
        QObject::connect(pushButton1, &QPushButton::clicked, Statistics, qOverload<>(&QDialog::reject));
        QObject::connect(pushButton2, &QPushButton::clicked, Statistics, qOverload<>(&QDialog::accept));

        QMetaObject::connectSlotsByName(Statistics);
    } // setupUi

    void retranslateUi(QDialog *Statistics)
    {
        Statistics->setWindowTitle(tr2i18n("Statistics", nullptr));
        p1_name->setText(tr2i18n("Player 1", nullptr));
        Name->setText(tr2i18n("Name", nullptr));
        won->setText(tr2i18n("Won", nullptr));
        lost->setText(tr2i18n("Lost", nullptr));
        sum->setText(tr2i18n("Sum", nullptr));
        aborted->setText(tr2i18n("Aborted", nullptr));
        pushButton1->setText(tr2i18n("Clear All Statistics", nullptr));
        pushButton2->setText(tr2i18n("&Close", nullptr));
        p2_name->setText(tr2i18n("Player 2", nullptr));
        drawn->setText(tr2i18n("Drawn", nullptr));
    } // retranslateUi

};

namespace Ui {
    class Statistics: public Ui_Statistics {};
} // namespace Ui

QT_END_NAMESPACE

#endif // STATISTICS_H

