/****************************************************************************
** Meta object code from reading C++ file 'kwin4.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kfourinline-26.07.90/src/kwin4.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kwin4.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8KWin4AppE_t {};
} // unnamed namespace

template <> constexpr inline auto KWin4App::qt_create_metaobjectdata<qt_meta_tag_ZN8KWin4AppE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KWin4App",
        "serverTypeChanged",
        "",
        "t",
        "remoteChanged",
        "who",
        "slotGameOver",
        "status",
        "KPlayer*",
        "p",
        "KGame*",
        "me",
        "moveDone",
        "playerNumber",
        "networkBroken",
        "id",
        "oldstatus",
        "game",
        "gameRun",
        "updateStatusNames",
        "configureNetwork",
        "configureChat",
        "debugKGame",
        "menuNewGame",
        "menuOpenGame",
        "menuSaveGame",
        "endGame",
        "showStatistics",
        "askForHint",
        "changeTheme",
        "idx",
        "undoMove",
        "redoMove",
        "displayStatusbarMover",
        "text",
        "displayStatusMessage",
        "configureSettings",
        "quickStart",
        "COLOUR",
        "startPlayer",
        "KGameIO::IOMode",
        "input0",
        "input1",
        "level"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'serverTypeChanged'
        QtMocHelpers::SlotData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'remoteChanged'
        QtMocHelpers::SlotData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'slotGameOver'
        QtMocHelpers::SlotData<void(int, KPlayer *, KGame *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 }, { 0x80000000 | 8, 9 }, { 0x80000000 | 10, 11 },
        }}),
        // Slot 'moveDone'
        QtMocHelpers::SlotData<void(int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 13 },
        }}),
        // Slot 'networkBroken'
        QtMocHelpers::SlotData<void(int, int, KGame *)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 15 }, { QMetaType::Int, 16 }, { 0x80000000 | 10, 17 },
        }}),
        // Slot 'gameRun'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateStatusNames'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'configureNetwork'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'configureChat'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'debugKGame'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuNewGame'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuOpenGame'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuSaveGame'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'endGame'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showStatistics'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'askForHint'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'changeTheme'
        QtMocHelpers::SlotData<void(int)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 30 },
        }}),
        // Slot 'undoMove'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'redoMove'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'displayStatusbarMover'
        QtMocHelpers::SlotData<void(const QString &)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 34 },
        }}),
        // Slot 'displayStatusMessage'
        QtMocHelpers::SlotData<void(const QString &)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 34 },
        }}),
        // Slot 'configureSettings'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'quickStart'
        QtMocHelpers::SlotData<void(COLOUR, KGameIO::IOMode, KGameIO::IOMode, int)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 38, 39 }, { 0x80000000 | 40, 41 }, { 0x80000000 | 40, 42 }, { QMetaType::Int, 43 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KWin4App, qt_meta_tag_ZN8KWin4AppE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KWin4App::staticMetaObject = { {
    QMetaObject::SuperData::link<KXmlGuiWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8KWin4AppE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8KWin4AppE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8KWin4AppE_t>.metaTypes,
    nullptr
} };

void KWin4App::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KWin4App *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->serverTypeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->remoteChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->slotGameOver((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlayer*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KGame*>>(_a[3]))); break;
        case 3: _t->moveDone((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->networkBroken((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KGame*>>(_a[3]))); break;
        case 5: _t->gameRun(); break;
        case 6: _t->updateStatusNames(); break;
        case 7: _t->configureNetwork(); break;
        case 8: _t->configureChat(); break;
        case 9: _t->debugKGame(); break;
        case 10: _t->menuNewGame(); break;
        case 11: _t->menuOpenGame(); break;
        case 12: _t->menuSaveGame(); break;
        case 13: _t->endGame(); break;
        case 14: _t->showStatistics(); break;
        case 15: _t->askForHint(); break;
        case 16: _t->changeTheme((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 17: _t->undoMove(); break;
        case 18: _t->redoMove(); break;
        case 19: _t->displayStatusbarMover((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 20: _t->displayStatusMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 21: _t->configureSettings(); break;
        case 22: _t->quickStart((*reinterpret_cast<std::add_pointer_t<COLOUR>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KGameIO::IOMode>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KGameIO::IOMode>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        default: ;
        }
    }
}

const QMetaObject *KWin4App::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KWin4App::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8KWin4AppE_t>.strings))
        return static_cast<void*>(this);
    return KXmlGuiWindow::qt_metacast(_clname);
}

int KWin4App::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KXmlGuiWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 23;
    }
    return _id;
}
QT_WARNING_POP
