/********************************************************************************
** Form generated from reading UI file 'settings.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SETTINGS_H
#define UI_SETTINGS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_Settings
{
public:
    QGridLayout *gridLayout;
    QGroupBox *Difficulty;
    QVBoxLayout *vboxLayout;
    QSlider *kcfg_level;
    QHBoxLayout *hboxLayout;
    QLabel *textLabel1;
    QLabel *textLabel2;
    QCheckBox *kcfg_learning;
    QSpacerItem *verticalSpacer;
    QGroupBox *Player1;
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *Name1;
    QLabel *nameLabel1;
    QLineEdit *kcfg_Name1;
    QCheckBox *kcfg_startcolourred;
    QSpacerItem *horizontalSpacer_3;
    QHBoxLayout *Input0;
    QLabel *playerLabel1;
    QRadioButton *kcfg_input0mouse;
    QRadioButton *kcfg_input0key;
    QRadioButton *kcfg_input0ai;
    QSpacerItem *horizontalSpacer;
    QGroupBox *Player2;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *Name2;
    QLabel *nameLabel2;
    QLineEdit *kcfg_Name2;
    QCheckBox *kcfg_startcolouryellow;
    QSpacerItem *horizontalSpacer_4;
    QHBoxLayout *Input1;
    QLabel *playerLabel2;
    QRadioButton *kcfg_input1mouse;
    QRadioButton *kcfg_input1key;
    QRadioButton *kcfg_input1ai;
    QSpacerItem *horizontalSpacer_2;

    void setupUi(QWidget *Settings)
    {
        if (Settings->objectName().isEmpty())
            Settings->setObjectName("Settings");
        Settings->resize(804, 457);
        Settings->setWindowOpacity(1.000000000000000);
        gridLayout = new QGridLayout(Settings);
        gridLayout->setObjectName("gridLayout");
        Difficulty = new QGroupBox(Settings);
        Difficulty->setObjectName("Difficulty");
        vboxLayout = new QVBoxLayout(Difficulty);
        vboxLayout->setObjectName("vboxLayout");
        kcfg_level = new QSlider(Difficulty);
        kcfg_level->setObjectName("kcfg_level");
        kcfg_level->setMinimum(1);
        kcfg_level->setMaximum(10);
        kcfg_level->setPageStep(1);
        kcfg_level->setValue(3);
        kcfg_level->setOrientation(Qt::Horizontal);
        kcfg_level->setTickPosition(QSlider::TicksBelow);

        vboxLayout->addWidget(kcfg_level);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        textLabel1 = new QLabel(Difficulty);
        textLabel1->setObjectName("textLabel1");

        hboxLayout->addWidget(textLabel1);

        textLabel2 = new QLabel(Difficulty);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        hboxLayout->addWidget(textLabel2);


        vboxLayout->addLayout(hboxLayout);

        kcfg_learning = new QCheckBox(Difficulty);
        kcfg_learning->setObjectName("kcfg_learning");

        vboxLayout->addWidget(kcfg_learning);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(verticalSpacer);


        gridLayout->addWidget(Difficulty, 3, 0, 1, 2);

        Player1 = new QGroupBox(Settings);
        Player1->setObjectName("Player1");
        verticalLayout_4 = new QVBoxLayout(Player1);
        verticalLayout_4->setObjectName("verticalLayout_4");
        Name1 = new QHBoxLayout();
        Name1->setObjectName("Name1");
        nameLabel1 = new QLabel(Player1);
        nameLabel1->setObjectName("nameLabel1");

        Name1->addWidget(nameLabel1);

        kcfg_Name1 = new QLineEdit(Player1);
        kcfg_Name1->setObjectName("kcfg_Name1");

        Name1->addWidget(kcfg_Name1);

        kcfg_startcolourred = new QCheckBox(Player1);
        kcfg_startcolourred->setObjectName("kcfg_startcolourred");
        kcfg_startcolourred->setChecked(true);
        kcfg_startcolourred->setAutoExclusive(true);

        Name1->addWidget(kcfg_startcolourred);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        Name1->addItem(horizontalSpacer_3);


        verticalLayout_4->addLayout(Name1);

        Input0 = new QHBoxLayout();
        Input0->setObjectName("Input0");
        playerLabel1 = new QLabel(Player1);
        playerLabel1->setObjectName("playerLabel1");

        Input0->addWidget(playerLabel1);

        kcfg_input0mouse = new QRadioButton(Player1);
        kcfg_input0mouse->setObjectName("kcfg_input0mouse");
        kcfg_input0mouse->setChecked(true);
        kcfg_input0mouse->setAutoExclusive(true);

        Input0->addWidget(kcfg_input0mouse);

        kcfg_input0key = new QRadioButton(Player1);
        kcfg_input0key->setObjectName("kcfg_input0key");
        kcfg_input0key->setAutoExclusive(true);

        Input0->addWidget(kcfg_input0key);

        kcfg_input0ai = new QRadioButton(Player1);
        kcfg_input0ai->setObjectName("kcfg_input0ai");
        kcfg_input0ai->setAutoExclusive(true);

        Input0->addWidget(kcfg_input0ai);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        Input0->addItem(horizontalSpacer);


        verticalLayout_4->addLayout(Input0);


        gridLayout->addWidget(Player1, 1, 0, 1, 1);

        Player2 = new QGroupBox(Settings);
        Player2->setObjectName("Player2");
        QFont font;
        font.setPointSize(10);
        Player2->setFont(font);
        verticalLayout = new QVBoxLayout(Player2);
        verticalLayout->setObjectName("verticalLayout");
        Name2 = new QHBoxLayout();
        Name2->setObjectName("Name2");
        nameLabel2 = new QLabel(Player2);
        nameLabel2->setObjectName("nameLabel2");

        Name2->addWidget(nameLabel2);

        kcfg_Name2 = new QLineEdit(Player2);
        kcfg_Name2->setObjectName("kcfg_Name2");

        Name2->addWidget(kcfg_Name2);

        kcfg_startcolouryellow = new QCheckBox(Player2);
        kcfg_startcolouryellow->setObjectName("kcfg_startcolouryellow");
        kcfg_startcolouryellow->setAutoExclusive(true);

        Name2->addWidget(kcfg_startcolouryellow);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        Name2->addItem(horizontalSpacer_4);


        verticalLayout->addLayout(Name2);

        Input1 = new QHBoxLayout();
        Input1->setObjectName("Input1");
        playerLabel2 = new QLabel(Player2);
        playerLabel2->setObjectName("playerLabel2");

        Input1->addWidget(playerLabel2);

        kcfg_input1mouse = new QRadioButton(Player2);
        kcfg_input1mouse->setObjectName("kcfg_input1mouse");
        kcfg_input1mouse->setChecked(true);

        Input1->addWidget(kcfg_input1mouse);

        kcfg_input1key = new QRadioButton(Player2);
        kcfg_input1key->setObjectName("kcfg_input1key");

        Input1->addWidget(kcfg_input1key);

        kcfg_input1ai = new QRadioButton(Player2);
        kcfg_input1ai->setObjectName("kcfg_input1ai");

        Input1->addWidget(kcfg_input1ai);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        Input1->addItem(horizontalSpacer_2);


        verticalLayout->addLayout(Input1);


        gridLayout->addWidget(Player2, 2, 0, 1, 1);

        QWidget::setTabOrder(kcfg_level, kcfg_learning);

        retranslateUi(Settings);

        QMetaObject::connectSlotsByName(Settings);
    } // setupUi

    void retranslateUi(QWidget *Settings)
    {
        Difficulty->setTitle(tr2i18n("Computer Difficulty", nullptr));
#if QT_CONFIG(whatsthis)
        kcfg_level->setWhatsThis(tr2i18n("Change the strength of the computer player.", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel1->setText(tr2i18n("Easy", nullptr));
        textLabel2->setText(tr2i18n("Hard", nullptr));
        kcfg_learning->setText(tr2i18n("Use AI learning", nullptr));
        Player1->setTitle(tr2i18n("Red", nullptr));
        nameLabel1->setText(tr2i18n("Name:", nullptr));
        kcfg_startcolourred->setText(tr2i18n("Starting Player", nullptr));
#if QT_CONFIG(whatsthis)
        playerLabel1->setWhatsThis(QString());
#endif // QT_CONFIG(whatsthis)
        playerLabel1->setText(tr2i18n("Player:", nullptr));
        kcfg_input0mouse->setText(tr2i18n("Human with Mouse", nullptr));
        kcfg_input0key->setText(tr2i18n("Human with Keyboard", nullptr));
        kcfg_input0ai->setText(tr2i18n("Computer", nullptr));
        Player2->setTitle(tr2i18n("Black", nullptr));
        nameLabel2->setText(tr2i18n("Name:", nullptr));
        kcfg_startcolouryellow->setText(tr2i18n("Starting Player", nullptr));
#if QT_CONFIG(whatsthis)
        playerLabel2->setWhatsThis(QString());
#endif // QT_CONFIG(whatsthis)
        playerLabel2->setText(tr2i18n("Player:", nullptr));
        kcfg_input1mouse->setText(tr2i18n("Human with Mouse", nullptr));
        kcfg_input1key->setText(tr2i18n("Human with Keyboard", nullptr));
        kcfg_input1ai->setText(tr2i18n("Computer", nullptr));
        (void)Settings;
    } // retranslateUi

};

namespace Ui {
    class Settings: public Ui_Settings {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SETTINGS_H

