/********************************************************************************
** Form generated from reading UI file 'statuswidget.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_STATUSWIDGET_H
#define UI_STATUSWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLCDNumber>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_StatusWidget
{
public:
    QGridLayout *gridLayout;
    QSpacerItem *spacerItem;
    QSpacerItem *spacerItem1;
    QLabel *wins;
    QLabel *draws;
    QLabel *loses;
    QLabel *num;
    QLabel *bk;
    QLabel *p1_name;
    QLCDNumber *p1_w;
    QLCDNumber *p1_d;
    QLCDNumber *p1_l;
    QLCDNumber *p1_n;
    QLCDNumber *p1_b;
    QLCDNumber *p2_d;
    QLCDNumber *p2_b;
    QLCDNumber *p2_l;
    QLCDNumber *p2_n;
    QLCDNumber *p2_w;
    QLabel *p2_name;
    QFrame *line1;

    void setupUi(QFrame *StatusWidget)
    {
        if (StatusWidget->objectName().isEmpty())
            StatusWidget->setObjectName("StatusWidget");
        StatusWidget->resize(305, 147);
        StatusWidget->setFrameShape(QFrame::Box);
        StatusWidget->setFrameShadow(QFrame::Raised);
        StatusWidget->setMidLineWidth(4);
        gridLayout = new QGridLayout(StatusWidget);
        gridLayout->setObjectName("gridLayout");
        spacerItem = new QSpacerItem(20, 41, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(spacerItem, 4, 5, 1, 1);

        spacerItem1 = new QSpacerItem(90, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacerItem1, 0, 0, 1, 1);

        wins = new QLabel(StatusWidget);
        wins->setObjectName("wins");
        wins->setAlignment(Qt::AlignCenter);

        gridLayout->addWidget(wins, 0, 1, 1, 1);

        draws = new QLabel(StatusWidget);
        draws->setObjectName("draws");
        draws->setAlignment(Qt::AlignCenter);

        gridLayout->addWidget(draws, 0, 2, 1, 1);

        loses = new QLabel(StatusWidget);
        loses->setObjectName("loses");
        loses->setAlignment(Qt::AlignCenter);

        gridLayout->addWidget(loses, 0, 3, 1, 1);

        num = new QLabel(StatusWidget);
        num->setObjectName("num");
        num->setAlignment(Qt::AlignCenter);

        gridLayout->addWidget(num, 0, 4, 1, 1);

        bk = new QLabel(StatusWidget);
        bk->setObjectName("bk");
        bk->setAlignment(Qt::AlignCenter);

        gridLayout->addWidget(bk, 0, 5, 1, 1);

        p1_name = new QLabel(StatusWidget);
        p1_name->setObjectName("p1_name");

        gridLayout->addWidget(p1_name, 2, 0, 1, 1);

        p1_w = new QLCDNumber(StatusWidget);
        p1_w->setObjectName("p1_w");
        p1_w->setLineWidth(0);
        p1_w->setDigitCount(2);

        gridLayout->addWidget(p1_w, 2, 1, 1, 1);

        p1_d = new QLCDNumber(StatusWidget);
        p1_d->setObjectName("p1_d");
        p1_d->setLineWidth(0);
        p1_d->setDigitCount(2);

        gridLayout->addWidget(p1_d, 2, 2, 1, 1);

        p1_l = new QLCDNumber(StatusWidget);
        p1_l->setObjectName("p1_l");
        p1_l->setLineWidth(0);
        p1_l->setDigitCount(2);

        gridLayout->addWidget(p1_l, 2, 3, 1, 1);

        p1_n = new QLCDNumber(StatusWidget);
        p1_n->setObjectName("p1_n");
        p1_n->setLineWidth(0);
        p1_n->setDigitCount(2);

        gridLayout->addWidget(p1_n, 2, 4, 1, 1);

        p1_b = new QLCDNumber(StatusWidget);
        p1_b->setObjectName("p1_b");
        p1_b->setLineWidth(0);
        p1_b->setDigitCount(2);

        gridLayout->addWidget(p1_b, 2, 5, 1, 1);

        p2_d = new QLCDNumber(StatusWidget);
        p2_d->setObjectName("p2_d");
        p2_d->setLineWidth(0);
        p2_d->setDigitCount(2);

        gridLayout->addWidget(p2_d, 3, 2, 1, 1);

        p2_b = new QLCDNumber(StatusWidget);
        p2_b->setObjectName("p2_b");
        p2_b->setLineWidth(0);
        p2_b->setDigitCount(2);

        gridLayout->addWidget(p2_b, 3, 5, 1, 1);

        p2_l = new QLCDNumber(StatusWidget);
        p2_l->setObjectName("p2_l");
        p2_l->setLineWidth(0);
        p2_l->setDigitCount(2);

        gridLayout->addWidget(p2_l, 3, 3, 1, 1);

        p2_n = new QLCDNumber(StatusWidget);
        p2_n->setObjectName("p2_n");
        p2_n->setLineWidth(0);
        p2_n->setDigitCount(2);

        gridLayout->addWidget(p2_n, 3, 4, 1, 1);

        p2_w = new QLCDNumber(StatusWidget);
        p2_w->setObjectName("p2_w");
        p2_w->setLineWidth(0);
        p2_w->setDigitCount(2);

        gridLayout->addWidget(p2_w, 3, 1, 1, 1);

        p2_name = new QLabel(StatusWidget);
        p2_name->setObjectName("p2_name");

        gridLayout->addWidget(p2_name, 3, 0, 1, 1);

        line1 = new QFrame(StatusWidget);
        line1->setObjectName("line1");
        line1->setFrameShape(QFrame::HLine);
        line1->setFrameShadow(QFrame::Raised);
        line1->setFrameShape(QFrame::Shape::HLine);

        gridLayout->addWidget(line1, 1, 0, 1, 6);


        retranslateUi(StatusWidget);

        QMetaObject::connectSlotsByName(StatusWidget);
    } // setupUi

    void retranslateUi(QFrame *StatusWidget)
    {
        wins->setText(tr2i18n("W", nullptr));
        draws->setText(tr2i18n("D", nullptr));
        loses->setText(tr2i18n("L", nullptr));
        num->setText(tr2i18n("No", nullptr));
        bk->setText(tr2i18n("Bk", nullptr));
        p1_name->setText(tr2i18n("Player 1", nullptr));
        p2_name->setText(tr2i18n("Player 2", nullptr));
        (void)StatusWidget;
    } // retranslateUi

};

namespace Ui {
    class StatusWidget: public Ui_StatusWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // STATUSWIDGET_H

