/****************************************************************************
** Meta object code from reading C++ file 'kwin4doc.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kfourinline-26.07.90/src/kwin4doc.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kwin4doc.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8KWin4DocE_t {};
} // unnamed namespace

template <> constexpr inline auto KWin4Doc::qt_create_metaobjectdata<qt_meta_tag_ZN8KWin4DocE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KWin4Doc",
        "signalGameRun",
        "",
        "signalChatChanged",
        "KWin4Player*",
        "player",
        "signalNextPlayer",
        "playerNumber",
        "moveDone",
        "mode",
        "loadSettings",
        "repeatMove",
        "processAIHintCommand",
        "QDataStream&",
        "in",
        "KGameProcessIO*",
        "io",
        "processAICommand",
        "clientConnected",
        "cid",
        "KGame*",
        "me",
        "prepareAITurn",
        "stream",
        "b",
        "KGameIO*",
        "input",
        "bool*",
        "eatevent",
        "networkMessageUpdate",
        "id",
        "sender",
        "receiver",
        "playerPropertyChanged",
        "KGamePropertyBase*",
        "prop",
        "KPlayer*",
        "gamePropertyChanged",
        "receivedStderr",
        "s"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'signalGameRun'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'signalChatChanged'
        QtMocHelpers::SignalData<void(KWin4Player *)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'signalNextPlayer'
        QtMocHelpers::SignalData<void(int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'moveDone'
        QtMocHelpers::SlotData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'loadSettings'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'repeatMove'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'processAIHintCommand'
        QtMocHelpers::SlotData<void(QDataStream &, KGameProcessIO *)>(12, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { 0x80000000 | 15, 16 },
        }}),
        // Slot 'processAICommand'
        QtMocHelpers::SlotData<void(QDataStream &, KGameProcessIO *)>(17, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { 0x80000000 | 15, 16 },
        }}),
        // Slot 'clientConnected'
        QtMocHelpers::SlotData<void(quint32, KGame *)>(18, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::UInt, 19 }, { 0x80000000 | 20, 21 },
        }}),
        // Slot 'prepareAITurn'
        QtMocHelpers::SlotData<void(QDataStream &, bool, KGameIO *, bool *)>(22, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 13, 23 }, { QMetaType::Bool, 24 }, { 0x80000000 | 25, 26 }, { 0x80000000 | 27, 28 },
        }}),
        // Slot 'networkMessageUpdate'
        QtMocHelpers::SlotData<void(int, quint32, quint32)>(29, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 30 }, { QMetaType::UInt, 31 }, { QMetaType::UInt, 32 },
        }}),
        // Slot 'playerPropertyChanged'
        QtMocHelpers::SlotData<void(KGamePropertyBase *, KPlayer *)>(33, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 34, 35 }, { 0x80000000 | 36, 5 },
        }}),
        // Slot 'gamePropertyChanged'
        QtMocHelpers::SlotData<void(KGamePropertyBase *, KGame *)>(37, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 34, 35 }, { 0x80000000 | 20, 21 },
        }}),
        // Slot 'receivedStderr'
        QtMocHelpers::SlotData<void(const QString &)>(38, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 39 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KWin4Doc, qt_meta_tag_ZN8KWin4DocE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KWin4Doc::staticMetaObject = { {
    QMetaObject::SuperData::link<KGame::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8KWin4DocE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8KWin4DocE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8KWin4DocE_t>.metaTypes,
    nullptr
} };

void KWin4Doc::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KWin4Doc *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->signalGameRun(); break;
        case 1: _t->signalChatChanged((*reinterpret_cast<std::add_pointer_t<KWin4Player*>>(_a[1]))); break;
        case 2: _t->signalNextPlayer((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->moveDone((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->loadSettings(); break;
        case 5: _t->repeatMove(); break;
        case 6: _t->processAIHintCommand((*reinterpret_cast<std::add_pointer_t<QDataStream&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KGameProcessIO*>>(_a[2]))); break;
        case 7: _t->processAICommand((*reinterpret_cast<std::add_pointer_t<QDataStream&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KGameProcessIO*>>(_a[2]))); break;
        case 8: _t->clientConnected((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KGame*>>(_a[2]))); break;
        case 9: _t->prepareAITurn((*reinterpret_cast<std::add_pointer_t<QDataStream&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KGameIO*>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool*>>(_a[4]))); break;
        case 10: _t->networkMessageUpdate((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<quint32>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<quint32>>(_a[3]))); break;
        case 11: _t->playerPropertyChanged((*reinterpret_cast<std::add_pointer_t<KGamePropertyBase*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlayer*>>(_a[2]))); break;
        case 12: _t->gamePropertyChanged((*reinterpret_cast<std::add_pointer_t<KGamePropertyBase*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KGame*>>(_a[2]))); break;
        case 13: _t->receivedStderr((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KWin4Player* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KGameProcessIO* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KGameProcessIO* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KGame* >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KGameIO* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlayer* >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KGame* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KWin4Doc::*)()>(_a, &KWin4Doc::signalGameRun, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KWin4Doc::*)(KWin4Player * )>(_a, &KWin4Doc::signalChatChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KWin4Doc::*)(int )>(_a, &KWin4Doc::signalNextPlayer, 2))
            return;
    }
}

const QMetaObject *KWin4Doc::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KWin4Doc::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8KWin4DocE_t>.strings))
        return static_cast<void*>(this);
    return KGame::qt_metacast(_clname);
}

int KWin4Doc::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KGame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void KWin4Doc::signalGameRun()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KWin4Doc::signalChatChanged(KWin4Player * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KWin4Doc::signalNextPlayer(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
